@@ -1,19 +1,23 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <bonobo/gnome-bonobo.h>
+#include <bonobo/bonobo.h>
+
+typedef struct {
+	GtkWidget  *box;
+	char *component_name;
+} closure_t;
 
 static void
-button_clicked_cb (GtkWidget *button, gpointer data)
+button_clicked_cb (GtkWidget *button, closure_t *cl)
 {
-	GtkWidget *box = GTK_WIDGET (data);
-	GnomeBonoboWidget *bw;
+	GtkWidget *bw;
 
-	bw = gnome_bonobo_widget_new ("embeddable:paint-component-simple", NULL);
+	bw = bonobo_widget_new_subdoc (cl->component_name, NULL);
 
-	gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (bw), TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (cl->box), bw, TRUE, TRUE, 0);
 
-	gtk_widget_show_all (GTK_WIDGET (box));
+	gtk_widget_show_all (cl->box);
 }
 
 int
@@ -23,6 +27,12 @@ main (int argc, char **argv)
 	CORBA_Environment ev;
 	GtkWidget *box;
 	GtkWidget *button;
+	closure_t  cl;
+
+	if (argc == 2 && argv[1])
+		cl.component_name = argv[1];
+	else
+		cl.component_name = "embeddable:paint-component-simple";
 
 	CORBA_exception_init (&ev);
 
@@ -37,11 +47,11 @@ main (int argc, char **argv)
         gtk_window_set_policy(GTK_WINDOW(app), TRUE, TRUE, FALSE);
 
 	box = gtk_vbox_new (FALSE, 5);
+	cl.box = box;
 
 	button = gtk_button_new_with_label ("Add component");
-
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			    button_clicked_cb, box);
+			    GTK_SIGNAL_FUNC (button_clicked_cb), &cl);
 
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 
