@@ -10,12 +10,12 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <bonobo/gnome-bonobo.h>
+#include <bonobo.h>
 
 #include "minefield.h"
 
 typedef struct {
-	GnomeEmbeddable     *embeddable;
+	BonoboEmbeddable     *embeddable;
 
 	MineField           *data;
 } embeddable_data_t;
@@ -24,31 +24,40 @@ typedef struct {
  * The per-view data.
  */
 typedef struct {
-	GnomeView	    *view;
+	BonoboView	    *view;
 	embeddable_data_t   *embeddable_data;
 	
 	GtkMineFieldView    *mfield;
 } view_data_t;
 
 /*
- * This callback is invoked when the GnomeEmbeddable object
+ * This callback is invoked when the BonoboEmbeddable object
  * encounters a fatal CORBA exception.
  */
 static void
-embeddable_system_exception_cb (GnomeEmbeddable *embeddable, CORBA_Object corba_object,
+embeddable_system_exception_cb (BonoboEmbeddable *embeddable, CORBA_Object corba_object,
 				CORBA_Environment *ev, gpointer data)
 {
-	gnome_object_destroy (GNOME_OBJECT (embeddable));
+	bonobo_object_destroy (BONOBO_OBJECT (embeddable));
 }
 
 /*
  * The view encounters a fatal corba exception.
  */
 static void
-view_system_exception_cb (GnomeView *view, CORBA_Object corba_object,
+view_system_exception_cb (BonoboView *view, CORBA_Object corba_object,
 			  CORBA_Environment *ev, gpointer data)
 {
-	gnome_object_destroy (GNOME_OBJECT (view));
+	bonobo_object_destroy (BONOBO_OBJECT (view));
+}
+
+static void
+queue_redraw (BonoboView *view, void *data)
+{
+	view_data_t *view_data;
+
+	view_data = gtk_object_get_data (GTK_OBJECT (view), "view_data");
+	gtk_widget_queue_draw (GTK_WIDGET (view_data->mfield));
 }
 
 /*
@@ -58,19 +67,11 @@ view_system_exception_cb (GnomeView *view, CORBA_Object corba_object,
 static void
 embeddable_update_all_views (embeddable_data_t *embeddable_data)
 {
-	GnomeEmbeddable *embeddable;
-	GList *l;
+	BonoboEmbeddable *embeddable;
 
 	embeddable = embeddable_data->embeddable;
 
-	for (l = embeddable->views; l != NULL; l = l->next) {
-		view_data_t *view_data;
-		GnomeView *view;
-
-		view = GNOME_VIEW (l->data);
-		view_data = gtk_object_get_data (GTK_OBJECT (view), "view_data");
-		gtk_widget_queue_draw (GTK_WIDGET (view_data->mfield));
-	}
+	bonobo_embeddable_foreach_view (embeddable, queue_redraw, NULL);
 }
 
 /*
@@ -78,7 +79,7 @@ embeddable_update_all_views (embeddable_data_t *embeddable_data)
  * free up our ancillary view-centric data structures.
  */
 static void
-view_destroy_cb (GnomeView *view, gpointer data)
+view_destroy_cb (BonoboView *view, gpointer data)
 {
 	view_data_t *view_data = (view_data_t *) data;
 
@@ -87,7 +88,8 @@ view_destroy_cb (GnomeView *view, gpointer data)
 }
 
 static void
-view_new_game_ui_cb  (GnomeUIHandler *uih, void *user_data, char *path)
+view_new_game_ui_cb  (BonoboUIHandler *uih, void *user_data,
+		      const char *path)
 {
 	view_data_t *view_data = (view_data_t *) user_data;
 
@@ -96,7 +98,7 @@ view_new_game_ui_cb  (GnomeUIHandler *uih, void *user_data, char *path)
 }
 
 static void
-view_new_game_cb (GnomeView *view, const char *verb_name, void *user_data)
+view_new_game_cb (BonoboView *view, const char *verb_name, void *user_data)
 {
 	view_data_t *view_data = (view_data_t *) user_data;
 
@@ -111,19 +113,19 @@ view_new_game_cb (GnomeView *view, const char *verb_name, void *user_data)
 static void
 view_create_menus (view_data_t *view_data)
 {
-	GNOME_UIHandler remote_uih;
-	GnomeView *view = view_data->view;
-	GnomeUIHandler *uih;
+	Bonobo_UIHandler remote_uih;
+	BonoboView *view = view_data->view;
+	BonoboUIHandler *uih;
 
 	/*
-	 * Grab our GnomeUIHandler object.
+	 * Grab our BonoboUIHandler object.
 	 */
-	uih = gnome_view_get_ui_handler (view);
+	uih = bonobo_view_get_ui_handler (view);
 
 	/*
 	 * Get our container's UIHandler server.
 	 */
-	remote_uih = gnome_view_get_remote_ui_handler (view);
+	remote_uih = bonobo_view_get_remote_ui_handler (view);
 
 	/*
 	 * We have to deal gracefully with containers
@@ -133,24 +135,24 @@ view_create_menus (view_data_t *view_data)
 		return;
 
 	/*
-	 * Give our GnomeUIHandler object a reference to the
+	 * Give our BonoboUIHandler object a reference to the
 	 * container's UIhandler server.
 	 */
-	gnome_ui_handler_set_container (uih, remote_uih);
+	bonobo_ui_handler_set_container (uih, remote_uih);
 
 	/*
 	 * Create our menu entries.
 	 */
-	gnome_ui_handler_menu_new_subtree (uih, "/Game",
-					   N_("Game"),
-					   N_("Change the game"),
+	bonobo_ui_handler_menu_new_subtree (uih, "/Game",
+					   L_("_Game"),
+					   _("Change the game"),
 					   1,
-					   GNOME_UI_HANDLER_PIXMAP_NONE, NULL,
+					   BONOBO_UI_HANDLER_PIXMAP_NONE, NULL,
 					   0, (GdkModifierType)0);
 
-	gnome_ui_handler_menu_new_item    (uih, "/Game/NewGame",
-					   N_("New Game"), N_("Start a new game"), -1,
-					   GNOME_UI_HANDLER_PIXMAP_NONE, NULL, 0,
+	bonobo_ui_handler_menu_new_item    (uih,  "/Game/NewGame",
+					   L_("_New game"), _("Start a new game"), -1,
+					   BONOBO_UI_HANDLER_PIXMAP_NONE, NULL, 0,
 					   (GdkModifierType)0, view_new_game_ui_cb, (gpointer)view_data);
 }
 
@@ -160,19 +162,19 @@ view_create_menus (view_data_t *view_data)
 static void
 view_remove_menus (view_data_t *view_data)
 {
-	GnomeView *view = view_data->view;
-	GnomeUIHandler *uih;
+	BonoboView *view = view_data->view;
+	BonoboUIHandler *uih;
 
-	uih = gnome_view_get_ui_handler (view);
+	uih = bonobo_view_get_ui_handler (view);
 
-	gnome_ui_handler_unset_container (uih);
+	bonobo_ui_handler_unset_container (uih);
 }
 
 /*
- * Clean up our supplementary GnomeEmbeddable data sturctures.
+ * Clean up our supplementary BonoboEmbeddable data sturctures.
  */
 static void
-embeddable_destroy_cb (GnomeEmbeddable *embeddable, gpointer data)
+embeddable_destroy_cb (BonoboEmbeddable *embeddable, gpointer data)
 {
 	embeddable_data_t *embeddable_data = (embeddable_data_t *) data;
 
@@ -184,7 +186,7 @@ embeddable_destroy_cb (GnomeEmbeddable *embeddable, gpointer data)
 }
 
 static void
-view_activate_cb (GnomeView *view, gboolean activate, gpointer data)
+view_activate_cb (BonoboView *view, gboolean activate, gpointer data)
 {
 	view_data_t *view_data = (view_data_t *) data;
 
@@ -192,11 +194,11 @@ view_activate_cb (GnomeView *view, gboolean activate, gpointer data)
 	 * The ViewFrame has just asked the View (that's us) to be
 	 * activated or deactivated.  We must reply to the ViewFrame
 	 * and say whether or not we want our activation state to
-	 * change.  We are an acquiescent GnomeView, so we just agree
+	 * change.  We are an acquiescent BonoboView, so we just agree
 	 * with whatever the ViewFrame told us.  Most components
 	 * should behave this way.
 	 */
-	gnome_view_activate_notify (view, activate);
+	bonobo_view_activate_notify (view, activate);
 
 	/*
 	 * If we were just activated, we merge in our menu entries.
@@ -208,42 +210,21 @@ view_activate_cb (GnomeView *view, gboolean activate, gpointer data)
 		view_remove_menus (view_data);
 }
 
-/*
- * This callback is invoked when the container asks a view what size
- * it wants to be.
- */
-static void
-view_size_query_cb (GnomeView *view, int *desired_width, int *desired_height,
-		    gpointer data)
-{
-	view_data_t *view_data = (view_data_t *) data;
-	GtkRequisition req;
-
-	if (view_data && view_data->mfield) {
-		gtk_widget_size_request (GTK_WIDGET (view_data->mfield), &req);
-		*desired_width  = req.width;
-		*desired_height = req.height;
-	} else {
-		*desired_width  = -1;
-		*desired_height = -1;
-	}
-}
-
 static void update_views (GtkWidget *widget, view_data_t *view_data)
 {
 	/* FIXME: all but current view ! */
 	embeddable_update_all_views (view_data->embeddable_data);
 }
 
-static GnomeView *
-view_factory (GnomeEmbeddable *embeddable,
-	      const GNOME_ViewFrame view_frame,
+static BonoboView *
+view_factory (BonoboEmbeddable *embeddable,
+	      const Bonobo_ViewFrame view_frame,
 	      void *data)
 {
 	embeddable_data_t *embeddable_data = (embeddable_data_t *) data;
 	view_data_t       *view_data;
-	GnomeUIHandler    *uih;
-	GnomeView         *view;
+	BonoboUIHandler    *uih;
+	BonoboView         *view;
 	GtkWidget         *vbox;
 
 	/*
@@ -271,30 +252,23 @@ view_factory (GnomeEmbeddable *embeddable,
 	gtk_widget_show_all (vbox);
 
 	/*
-	 * Create the GnomeView object.
+	 * Create the BonoboView object.
 	 */
-	view = gnome_view_new (vbox);
+	view = bonobo_view_new (vbox);
 	view_data->view = view;
 	gtk_object_set_data (GTK_OBJECT (view), "view_data", view_data);
 
 	/*
-	 * Create the GnomeUIHandler for this view.  It will be used
+	 * Create the BonoboUIHandler for this view.  It will be used
 	 * to merge menu and toolbar items when the view is activated.
 	 */
-	uih = gnome_ui_handler_new ();
-	gnome_view_set_ui_handler (view, uih);
+	uih = bonobo_ui_handler_new ();
+	bonobo_view_set_ui_handler (view, uih);
 
 	/*
 	 * Register callbacks to handle the verbs
 	 */
-	gnome_view_register_verb (view, "NewGame", view_new_game_cb, view_data);
-
-	/*
-	 * The "size_query" signal is raised when the container asks
-	 * the component what size it wants to be.
-	 */
-	gtk_signal_connect (GTK_OBJECT (view), "size_query",
-			    GTK_SIGNAL_FUNC (view_size_query_cb), view_data);
+	bonobo_view_register_verb (view, "NewGame", view_new_game_cb, view_data);
 
 	/*
 	 * When our container wants to activate a given view of this
@@ -304,7 +278,7 @@ view_factory (GnomeEmbeddable *embeddable,
 			    GTK_SIGNAL_FUNC (view_activate_cb), view_data);
 
 	/*
-	 * The "system_exception" signal is raised when the GnomeView
+	 * The "system_exception" signal is raised when the BonoboView
 	 * encounters a fatal CORBA exception.
 	 */
 	gtk_signal_connect (GTK_OBJECT (view), "system_exception",
@@ -320,11 +294,11 @@ view_factory (GnomeEmbeddable *embeddable,
 	return view;
 }
 
-static GnomeObject *
-embeddable_factory (GnomeEmbeddableFactory *this,
+static BonoboObject *
+embeddable_factory (BonoboEmbeddableFactory *this,
 		    void *data)
 {
-	GnomeEmbeddable *embeddable;
+	BonoboEmbeddable *embeddable;
 	embeddable_data_t *embeddable_data;
 
 	/*
@@ -336,9 +310,9 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 		return NULL;
 
 	/*
-	 * Create the GnomeEmbeddable object.
+	 * Create the BonoboEmbeddable object.
 	 */
-	embeddable = gnome_embeddable_new (view_factory, embeddable_data);
+	embeddable = bonobo_embeddable_new (view_factory, embeddable_data);
 
 	if (embeddable == NULL) {
 		g_free (embeddable_data);
@@ -355,10 +329,10 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 	 * Add some verbs to the embeddable.
 	 *
 	 * Verbs are simple non-paramameterized actions which the
-	 * component can perform.  The GnomeEmbeddable must maintain a
+	 * component can perform.  The BonoboEmbeddable must maintain a
 	 * list of the verbs which a component supports, and the
 	 * component author must register callbacks for each of his
-	 * verbs with the GnomeView.
+	 * verbs with the BonoboView.
 	 *
 	 * The container application will then have the programmatic
 	 * ability to execute the verbs on the component.  It will
@@ -369,7 +343,7 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 	 * We provide one simple verb whose job it is to clear the
 	 * window.
 	 */
-	gnome_embeddable_add_verb (embeddable,
+	bonobo_embeddable_add_verb (embeddable,
 				   "NewGame",
 				   _("_New Game"),
 				   _("Start a new game"));
@@ -379,7 +353,7 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 	 * will emit a "system_exception" signal, notifying us that
 	 * the object is defunct.  Our callback --
 	 * embeddable_system_exception_cb() -- destroys the defunct
-	 * GnomeEmbeddable object.
+	 * BonoboEmbeddable object.
 	 */
 	gtk_signal_connect (GTK_OBJECT (embeddable), "system_exception",
 			    GTK_SIGNAL_FUNC (embeddable_system_exception_cb),
@@ -394,15 +368,15 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 			    GTK_SIGNAL_FUNC (embeddable_destroy_cb),
 			    embeddable_data);
 
-	return GNOME_OBJECT (embeddable);
+	return BONOBO_OBJECT (embeddable);
 }
 
 static void
 init_simple_mines_factory (void)
 {
-	GnomeEmbeddableFactory *factory;
+	BonoboEmbeddableFactory *factory;
 
-	factory = gnome_embeddable_factory_new (
+	factory = bonobo_embeddable_factory_new (
 		"embeddable-factory:application-x-mines",
 		embeddable_factory, NULL);
 }
@@ -413,6 +387,9 @@ init_server_factory (int argc, char **argv)
 	CORBA_Environment ev;
 	CORBA_ORB orb;
 
+        bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	textdomain (PACKAGE);
+
 	CORBA_exception_init (&ev);
 
 	gnome_CORBA_init_with_popt_table (
