@@ -8,7 +8,7 @@
 #include <config.h>
 #include <gnome.h>
 #include <libgnorba/gnorba.h>
-#include <bonobo/gnome-bonobo.h>
+#include <bonobo.h>
 
 /* XPM */
 static char * bolt_xpm[] = {
@@ -36,14 +36,21 @@ static char * bolt_xpm[] = {
 ".+@.      ",
 "...       "};
 
-CORBA_Environment ev;
-CORBA_ORB orb;
+/*
+ * Number of running objects
+ */ 
+static int running_objects = 0;
+
+/*
+ * Our embeddable factory
+ */
+static BonoboEmbeddableFactory *factory;
 
 /*
  * BonoboObject data
  */
 typedef struct {
-	GnomeEmbeddable *bonobo_object;
+	BonoboEmbeddable *bonobo_object;
 
 	gchar             *text;
 	guint              text_len;
@@ -57,7 +64,7 @@ typedef struct {
 	 */
 	size_t             total_size;
 
-	/* List of this component's GNOME::Views. */
+	/* List of this component's Bonobo::Views. */
 	GList *views;
 } bonobo_object_data_t;
 
@@ -65,8 +72,7 @@ typedef struct {
  * View data
  */
 typedef struct {
-	GnomeView        *view;
-	GnomeUIHandler	 *uih;
+	BonoboView        *view;
 
 	bonobo_object_data_t *bonobo_object_data;
 
@@ -252,34 +258,46 @@ progressive_update (bonobo_object_data_t *bonobo_object_data,
  * Callbacks attached to signals.
  */
 static gboolean
-view_destroy_cb (GnomeView *view_ignored, view_data_t *view_data)
+view_destroy_cb (BonoboView *view_ignored, view_data_t *view_data)
 {
 	destroy_view (view_data);
 
 	return FALSE;
 } /* view_destroy_cb */
 
-static gboolean
-bonobo_object_destroy_cb (GnomeEmbeddable *bonobo_object,
+static void
+view_system_exception_cb (BonoboView *view, CORBA_Object corba_object,
+			  CORBA_Environment *ev, gpointer data)
+{
+	g_warning ("View system exception");
+	bonobo_object_destroy (BONOBO_OBJECT (view));
+}
+
+static void
+embeddable_system_exception_cb (BonoboEmbeddable *embeddable, CORBA_Object corba_object,
+				CORBA_Environment *ev, gpointer data)
+{
+	g_warning ("Embeddable system exception");
+	bonobo_object_destroy (BONOBO_OBJECT (embeddable));
+}
+
+static void
+bonobo_object_destroy_cb (BonoboEmbeddable *bonobo_object,
 			  bonobo_object_data_t *bonobo_object_data)
 {
-	/*
-	 * Destroy all views.
-	 */
-/*	while (bonobo_object_data->views) {
-		view_data_t *view_data = bonobo_object_data->views->data;
-
-		destroy_view (view_data);
-		}*/
-
-	/*
-	 * Destroy the BonoboObject.
-	 */
 	free_text (bonobo_object_data);
 	g_free (bonobo_object_data);
 
-	return FALSE;
-} /* bonobo_object_destroy_cb */
+	running_objects--;
+	if (running_objects != 0)
+		return;
+
+	if (factory) {
+		bonobo_object_unref (BONOBO_OBJECT (factory));
+		factory = NULL;
+		gtk_main_quit ();
+	}
+}
 
 
 static gboolean
@@ -384,7 +402,8 @@ text_deleted_cb (GtkText *text,
 } /* text_deleted_cb */
 
 static gint
-clear_text_cb (GnomeUIHandler *uih, void *data, char *path)
+clear_text_cb (BonoboUIHandler *uih, void *data,
+	       const char *path)
 {
 	view_data_t *view_data = (view_data_t *) data;
 
@@ -405,52 +424,53 @@ clear_text_cb (GnomeUIHandler *uih, void *data, char *path)
 static void
 create_view_menus (view_data_t *view_data)
 {
-	GNOME_UIHandler remote_uih;
-	CORBA_Environment ev;
+	Bonobo_UIHandler remote_uih;
+	BonoboUIHandler *uih;
 	char label[256];
 
 	/*
 	 * Register with the UI Handler and add some menu entries.
 	 */
-	remote_uih = gnome_view_get_ui_handler (view_data->view);
+	uih = bonobo_view_get_ui_handler (view_data->view);
+	remote_uih = bonobo_view_get_remote_ui_handler (view_data->view);
 	if (remote_uih == CORBA_OBJECT_NIL)
 		return;
 
-	gnome_ui_handler_set_container (view_data->uih, remote_uih);
-	
-	CORBA_exception_init (&ev);
-	CORBA_Object_release (remote_uih, &ev);
-	CORBA_exception_free (&ev);
-	
-	gnome_ui_handler_menu_new_separator (view_data->uih, "/text\\/plain/separator1", -1);
+	bonobo_ui_handler_set_container (uih, remote_uih);
+		
+	bonobo_ui_handler_menu_new_separator (uih, "/text\\/plain/separator1", -1);
 
 	snprintf (label, sizeof (label), "Clear Text (containee-added item for view %p)", view_data);
-	gnome_ui_handler_menu_new_item (view_data->uih, "/text\\/plain/Clear text", 
-					label, NULL, -1, GNOME_UI_HANDLER_PIXMAP_XPM_DATA,
+	bonobo_ui_handler_menu_new_item (uih, "/text\\/plain/Clear text", 
+					label, NULL, -1, BONOBO_UI_HANDLER_PIXMAP_XPM_DATA,
 					bolt_xpm, 0, 0, clear_text_cb, view_data);
 
-	gnome_ui_handler_menu_new_item (view_data->uih, "/text\\/plain/Clear text", 
-					label, NULL, -1, GNOME_UI_HANDLER_PIXMAP_XPM_DATA,
+	bonobo_ui_handler_menu_new_item (uih, "/text\\/plain/Clear text", 
+					label, NULL, -1, BONOBO_UI_HANDLER_PIXMAP_XPM_DATA,
 					bolt_xpm, 0, 0, clear_text_cb, view_data);
 
-	gnome_ui_handler_create_toolbar (view_data->uih, "Common");
+	bonobo_ui_handler_create_toolbar (uih, "Common");
 
-	gnome_ui_handler_toolbar_new_item (view_data->uih, "/Common/item1", "item 1", NULL, -1,
-					   GNOME_UI_HANDLER_PIXMAP_XPM_DATA, bolt_xpm,
+	bonobo_ui_handler_toolbar_new_item (uih, "/Common/item1", "item 1", NULL, -1,
+					   BONOBO_UI_HANDLER_PIXMAP_XPM_DATA, bolt_xpm,
 					   0, 0, NULL, NULL);
 }
 
 static void
 destroy_view_menus (view_data_t *view_data)
 {
-	gnome_ui_handler_unset_container (view_data->uih);
+	BonoboView *view = view_data->view;
+	BonoboUIHandler *uih;
+
+	uih = bonobo_view_get_ui_handler (view);
+
+	bonobo_ui_handler_unset_container (uih);
 }
 
 static void
-view_activate_cb (GnomeView *view, gboolean activate, gpointer data)
+view_activate_cb (BonoboView *view, gboolean activate, gpointer data)
 {
 	view_data_t *view_data = (view_data_t *) data;
-	CORBA_Environment ev;
 
 	/*
 	 * If we have been activated, create our menus.  If we have
@@ -463,31 +483,31 @@ view_activate_cb (GnomeView *view, gboolean activate, gpointer data)
 
 	/*
 	 * Notify the ViewFrame that we accept to be activated or
-	 * deactivated (we are an acquiescent GnomeView, yes we are).
+	 * deactivated (we are an acquiescent BonoboView, yes we are).
 	 */
-	GNOME_ViewFrame_view_activated (gnome_view_get_view_frame (view), activate, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		/* FIXME: Do something I suppose. */
-	}
+	bonobo_view_activate_notify (view, activate);
 }
 
 /*
- * GNOME::PersistStream
+ * Bonobo::PersistStream
  *
- * These two functions implement the GNOME::PersistStream load and
+ * These two functions implement the Bonobo::PersistStream load and
  * save methods which allow data to be loaded into and out of the
  * BonoboObject.
  */
 static int
-stream_read (GNOME_Stream stream, bonobo_object_data_t *bonobo_object_data)
+stream_read (Bonobo_Stream stream, bonobo_object_data_t *bonobo_object_data)
 {
-	GNOME_Stream_iobuf *buffer;
+	Bonobo_Stream_iobuf *buffer;
 	CORBA_long bytes_read;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
 
 	do {
 
 #define READ_CHUNK_SIZE 65536
-		bytes_read = GNOME_Stream_read (stream, READ_CHUNK_SIZE,
+		bytes_read = Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
 						&buffer, &ev);
 
 		bonobo_object_data->text = g_realloc (bonobo_object_data->text,
@@ -503,6 +523,7 @@ stream_read (GNOME_Stream stream, bonobo_object_data_t *bonobo_object_data)
 		CORBA_free (buffer);
 	} while (bytes_read > 0);
 
+	CORBA_exception_free (&ev);
 
 	if (bytes_read < 0)
 		return -1;
@@ -511,10 +532,10 @@ stream_read (GNOME_Stream stream, bonobo_object_data_t *bonobo_object_data)
 } /* stream_read */
 
 /*
- * This function implements the GNOME::PersistStream:load method.
+ * This function implements the Bonobo::PersistStream:load method.
  */
 static int
-pstream_load (GnomePersistStream *ps, GNOME_Stream stream,
+pstream_load (BonoboPersistStream *ps, Bonobo_Stream stream,
 	      void *data)
 {
 	bonobo_object_data_t *bonobo_object_data = data;
@@ -540,22 +561,25 @@ pstream_load (GnomePersistStream *ps, GNOME_Stream stream,
 } /* pstream_load */
 
 /*
- * This function implements the GNOME::PersistStream:save method.
+ * This function implements the Bonobo::PersistStream:save method.
  */
 static int
-pstream_save (GnomePersistStream *ps, GNOME_Stream stream,
+pstream_save (BonoboPersistStream *ps, Bonobo_Stream stream,
 	      void *data)
 {
 	bonobo_object_data_t *bonobo_object_data = data;
-	GNOME_Stream_iobuf *buffer;
+	Bonobo_Stream_iobuf *buffer;
 	size_t pos;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
 	
 	/*
 	 * Write the text data into the stream.
 	 *
 	 * FIXME: Do we really _have_ to double-buffer the text?
 	 */
-	buffer = GNOME_Stream_iobuf__alloc ();
+	buffer = Bonobo_Stream_iobuf__alloc ();
 
 	data = CORBA_sequence_CORBA_octet_allocbuf (bonobo_object_data->text_len);
 	memcpy (data, bonobo_object_data->text, bonobo_object_data->text_len);
@@ -567,11 +591,12 @@ pstream_save (GnomePersistStream *ps, GNOME_Stream stream,
 	while (pos < bonobo_object_data->text_len) {
 		CORBA_long bytes_read;
 
-		bytes_read = GNOME_Stream_write (stream, buffer, &ev);
+		bytes_read = Bonobo_Stream_write (stream, buffer, &ev);
 
 		if (ev._major != CORBA_NO_EXCEPTION) {
 			CORBA_free (buffer);
 			CORBA_free (data);
+			CORBA_exception_free (&ev);
 			return -1;
 		}
 
@@ -580,12 +605,13 @@ pstream_save (GnomePersistStream *ps, GNOME_Stream stream,
 
 	CORBA_free (buffer);
 	CORBA_free (data);
+	CORBA_exception_free (&ev);
 	
 	return 0;
 } /* pstream_save */
 
 /*
- * GNOME::ProgressiveDataSink
+ * Bonobo::ProgressiveDataSink
  *
  * These functions implement the ProgressiveDataSink interface
  * methods, which are used to send a slow stream of data to the widget
@@ -593,7 +619,7 @@ pstream_save (GnomePersistStream *ps, GNOME_Stream stream,
  */
 
 static int
-progressive_start (GnomeProgressiveDataSink *psink, void *data)
+progressive_start (BonoboProgressiveDataSink *psink, void *data)
 {
 	bonobo_object_data_t * bonobo_object_data =
 		(bonobo_object_data_t *) data;
@@ -619,7 +645,7 @@ progressive_start (GnomeProgressiveDataSink *psink, void *data)
 } /* progressive_start */
 
 static int
-progressive_end (GnomeProgressiveDataSink *psink, void *data)
+progressive_end (BonoboProgressiveDataSink *psink, void *data)
 {
 	bonobo_object_data_t * bonobo_object_data =
 		(bonobo_object_data_t *) data;
@@ -640,7 +666,7 @@ progressive_end (GnomeProgressiveDataSink *psink, void *data)
 } /* progressive_end */
 
 static int
-progressive_set_size (GnomeProgressiveDataSink *psink,
+progressive_set_size (BonoboProgressiveDataSink *psink,
 		      CORBA_long size, void *data)
 {
 	bonobo_object_data_t * bonobo_object_data =
@@ -652,8 +678,8 @@ progressive_set_size (GnomeProgressiveDataSink *psink,
 } /* progressive_set_size */
 
 static int
-progressive_add_data (GnomeProgressiveDataSink *psink,
-		      const GNOME_ProgressiveDataSink_iobuf *buffer,
+progressive_add_data (BonoboProgressiveDataSink *psink,
+		      const Bonobo_ProgressiveDataSink_iobuf *buffer,
 		      void *data)
 {
 	bonobo_object_data_t *bonobo_object_data =
@@ -668,12 +694,13 @@ progressive_add_data (GnomeProgressiveDataSink *psink,
 /*
  * BonoboObject construction functions.
  */
-static GnomeView *
-view_factory (GnomeEmbeddable *bonobo_object,
-	      const GNOME_ViewFrame view_frame,
+static BonoboView *
+view_factory (BonoboEmbeddable *bonobo_object,
+	      const Bonobo_ViewFrame view_frame,
 	      void *data)
 {
-        GnomeView *view;
+        BonoboView *view;
+	BonoboUIHandler *uih;
 	bonobo_object_data_t *bonobo_object_data = data;
 	view_data_t *view_data;
 
@@ -683,8 +710,6 @@ view_factory (GnomeEmbeddable *bonobo_object,
 	view_data = g_new0 (view_data_t, 1);
 	view_data->bonobo_object_data = bonobo_object_data;
 
-	view_data->uih = gnome_ui_handler_new ();
-
 	view_data->text = gtk_text_new (NULL, NULL);
 
 	/*
@@ -732,7 +757,7 @@ view_factory (GnomeEmbeddable *bonobo_object,
 	/*
 	 * Now create the GtkProgress bar which will be displayed when
 	 * data is being loaded into the BonoboObject with the
-	 * GNOME::ProgressiveDataSink interface.
+	 * Bonobo::ProgressiveDataSink interface.
 	 */
 	view_data->progress = gtk_progress_bar_new ();
 	gtk_table_attach (GTK_TABLE (view_data->table), view_data->progress,
@@ -746,16 +771,22 @@ view_factory (GnomeEmbeddable *bonobo_object,
 		gtk_widget_hide (view_data->progress);
 
 	/*
-	 * Now create the GnomeView object.
+	 * Now create the BonoboView object.
 	 */
-        view = gnome_view_new (view_data->table);
+        view = bonobo_view_new (view_data->table);
 	view_data->view = view;
 
-	gnome_view_set_view_frame (view, view_frame);
+	uih = bonobo_ui_handler_new ();
+	bonobo_view_set_ui_handler (view, uih);
+
+	bonobo_view_set_view_frame (view, view_frame);
 
 	gtk_signal_connect (GTK_OBJECT (view), "destroy",
 			    GTK_SIGNAL_FUNC (view_destroy_cb), view_data);
 
+	gtk_signal_connect (GTK_OBJECT (view), "system_exception",
+			    GTK_SIGNAL_FUNC (view_system_exception_cb), view_data);
+
 	gtk_signal_connect (GTK_OBJECT (view), "view_activate",
 			    GTK_SIGNAL_FUNC (view_activate_cb), view_data);
 
@@ -765,13 +796,13 @@ view_factory (GnomeEmbeddable *bonobo_object,
         return view;
 } /* view_factory */
 
-static GnomeObject *
-embeddable_factory (GnomeEmbeddableFactory *this,
+static BonoboObject *
+embeddable_factory (BonoboEmbeddableFactory *this,
 		    void *data)
 {
-	GnomeEmbeddable *bonobo_object;
-	GnomePersistStream *stream;
-	GnomeProgressiveDataSink *psink;
+	BonoboEmbeddable *bonobo_object;
+	BonoboPersistStream *stream;
+	BonoboProgressiveDataSink *psink;
 	bonobo_object_data_t *bonobo_object_data;
 
 	bonobo_object_data = g_new0 (bonobo_object_data_t, 1);
@@ -784,40 +815,44 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 	/*
 	 * Creates the BonoboObject server
 	 */
-	bonobo_object = gnome_embeddable_new (view_factory,
+	bonobo_object = bonobo_embeddable_new (view_factory,
 					      bonobo_object_data);
-	if (bonobo_object == NULL){
+	if (bonobo_object == NULL) {
 		g_free (bonobo_object_data);
 		return NULL;
 	}
+	running_objects++;
 
 	gtk_signal_connect (GTK_OBJECT (bonobo_object), "destroy",
 			    GTK_SIGNAL_FUNC (bonobo_object_destroy_cb),
 			    bonobo_object_data);
 
+	gtk_signal_connect (GTK_OBJECT (bonobo_object), "system_exception",
+			    GTK_SIGNAL_FUNC (embeddable_system_exception_cb),
+			    bonobo_object_data);
+
 	bonobo_object_data->bonobo_object = bonobo_object;
 
 	/*
-	 * Register the GNOME::PersistStream interface.
+	 * Register the Bonobo::PersistStream interface.
 	 */
-	stream = gnome_persist_stream_new ("bonobo-object:text-plain",
-					   pstream_load,
+	stream = bonobo_persist_stream_new (pstream_load,
 					   pstream_save,
 					   bonobo_object_data);
-	if (stream == NULL){
+	if (stream == NULL) {
 		gtk_object_unref (GTK_OBJECT (bonobo_object));
 		g_free (bonobo_object_data);
 		return NULL;
 	}
 
-	gnome_object_add_interface (GNOME_OBJECT (bonobo_object),
-				    GNOME_OBJECT (stream));
+	bonobo_object_add_interface (BONOBO_OBJECT (bonobo_object),
+				    BONOBO_OBJECT (stream));
 
 
 	/*
-	 * Register the GNOME::ProgressiveDataSink interface.
+	 * Register the Bonobo::ProgressiveDataSink interface.
 	 */
-	psink = gnome_progressive_data_sink_new (progressive_start,
+	psink = bonobo_progressive_data_sink_new (progressive_start,
 						 progressive_end,
 						 progressive_add_data,
 						 progressive_set_size,
@@ -828,18 +863,16 @@ embeddable_factory (GnomeEmbeddableFactory *this,
 		return NULL;
 	}
 
-	gnome_object_add_interface (GNOME_OBJECT (bonobo_object),
-				    GNOME_OBJECT (psink));
+	bonobo_object_add_interface (BONOBO_OBJECT (bonobo_object),
+				    BONOBO_OBJECT (psink));
 
-	return GNOME_OBJECT (bonobo_object);
+	return BONOBO_OBJECT (bonobo_object);
 } /* embeddable_factory */
 
 static void
 init_bonobo_text_plain_factory (void)
 {
-	GnomeEmbeddableFactory *factory;
-	
-	factory = gnome_embeddable_factory_new (
+	factory = bonobo_embeddable_factory_new (
 		"embeddable-factory:text-plain",
 		embeddable_factory, NULL);
 } /* init_bonobo_text_plain_factory */
@@ -847,6 +880,11 @@ init_bonobo_text_plain_factory (void)
 static void
 init_server_factory (int argc, char **argv)
 {
+	CORBA_ORB orb;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+	
 	gnome_CORBA_init_with_popt_table (
 		"bonobo-text-plain", VERSION,
 		&argc, argv, NULL, 0, NULL, GNORBA_INIT_SERVER_FUNC, &ev);
@@ -854,19 +892,17 @@ init_server_factory (int argc, char **argv)
 	orb = gnome_CORBA_ORB ();
 	if (bonobo_init (orb, NULL, NULL) == FALSE)
 		g_error (_("I could not initialize Bonobo"));
+
+	CORBA_exception_free (&ev);
 } /* init_server_factory */
 
 int
 main (int argc, char **argv)
 {
-	CORBA_exception_init (&ev);
-
 	init_server_factory (argc, argv);
 	init_bonobo_text_plain_factory ();
 
 	bonobo_main ();
 
-	CORBA_exception_free (&ev);
-
 	return 0;
 } /* main */
