@@ -17,7 +17,7 @@
 
 #include "vefs.h"
 
-static EFSDriver *efs_driver_list[] = { 
+EFSDriver *efs_driver_list[] = { 
 	&efs_driver_simple, 
 	&efs_driver_fsys,
 	NULL 
@@ -85,7 +85,7 @@ efs_open (const char *path, gint flags, gint mode)
 	g_return_val_if_fail (path != NULL, NULL);
 
 	/* extract driver name */
-	i = 0; while ((i<1024)&&path[i]&&isalpha(path[i])) i++;
+	i = 0; while ((i<1024)&&path[i]&&isalpha((int)path[i])) i++;
 	filename = path;
 	drivername[0] = 0;
 
@@ -100,6 +100,8 @@ efs_open (const char *path, gint flags, gint mode)
 	if (!(stat (filename, &sb))) {       /* exists */
 		if ((flags&EFS_CREATE)&&(flags&EFS_EXCL)) return NULL;
 		if (S_ISDIR(sb.st_mode)) {   /* use fsys: driver */
+			if (drivername[0] && 
+			    strcmp(drivername,"fsys")) return NULL;
 			if (!(d = efs_find_driver("fsys"))) return NULL;
 		} else {                     /* detect driver */
 			if ((flags&EFS_CREATE)||
@@ -149,18 +151,14 @@ efs_close (EFSDir *root)
 {
 	EFSDriver *d;
 	gint rval;
-
 	g_return_val_if_fail (root != NULL, -1);
 
 	d = root->efs->driver;
 
-	rval = d->sops->close (root->efs);
-
-	flock (root->efs->fd, LOCK_UN);
-	close (root->efs->fd);
-
-	efs_free (root->efs);
-	g_free (root);
+	if (!(rval = d->sops->close (root->efs))) {
+		efs_free (root->efs);
+		g_free (root);
+	}
 
 	return rval;
 }
